/**
Dieser Test berechnet Beispiele wie sie in "Primary Directions - 
A Primer of Calculation" von Bob Makransky abgedruckt sind.
Siehe: www.dearbrutus.com
*/

#include <conio.h>
#include <stdio.h>
#include <math.h>
#include "munpos.h"

typedef struct {
  const char *obj;
  double lon, lat;
} PLAN;

static void deg2zod(double deg, char* s);
static void deg2dm(double deg, char *s);

int main()
{
  MUNPOS mp;
  SPECULUM spec;
  PLAN plan[16]; 
  double eps, st, geolon, geolat;
  double amp, kmp;
  int i, nplan;
  char pLon[16], pLat[16], pDec[16];
  char mi, dn, ew, zn;
  
  plan[ 0].obj = " SUN    "; plan[ 0].lon = 232.4230148217789633; plan[ 0].lat =  -0.0001036596952817;
  plan[ 1].obj = " MOON   "; plan[ 1].lon =  30.4367210107175872; plan[ 1].lat =  -0.4161147560392354;
  plan[ 2].obj = " MERCURY"; plan[ 2].lon = 216.9576912219833957; plan[ 2].lat =   1.7808243485431556;
  plan[ 3].obj = " VENUS  "; plan[ 3].lon = 196.3844819115678035; plan[ 3].lat =   1.8253532582353658;
  plan[ 4].obj = " MARS   "; plan[ 4].lon = 260.9489622185521966; plan[ 4].lat =  -0.9280320162394278;
  plan[ 5].obj = " JUPITER"; plan[ 5].lon = 269.8856259635734318; plan[ 5].lat =   0.0687041636217336;
  plan[ 6].obj = " SATURN "; plan[ 6].lon = 155.2674008465341444; plan[ 6].lat =   1.4390917591699042;
  plan[ 7].obj = " URANUS "; plan[ 7].lon =  89.9290134901287814; plan[ 7].lat =   0.1997064150055768;
  plan[ 8].obj = " NEPTUNE"; plan[ 8].lon = 194.1290302643338066; plan[ 8].lat =   1.5244997092861730;
  plan[ 9].obj = " PLUTO  "; plan[ 9].lon = 136.5628727509733551; plan[ 9].lat =   7.5570987540082148;
  plan[10].obj = " CHIRON "; plan[10].lon = 238.23;               plan[10].lat =   3.+10./60.;
  plan[11].obj = " COMET  "; plan[11].lon = 197.96;               plan[11].lat = -21.-13./60.;
  plan[12].obj = " NODE   "; plan[12].lon =  34.96;               plan[12].lat =           0.;
  plan[13].obj = " ASC    "; plan[13].lon = 125.50;               plan[13].lat =           0.;
  plan[14].obj = " MC     "; plan[14].lon =  13.44;               plan[14].lat =           0.;
  plan[15].obj = " ATX    "; plan[15].lon = 263.00;               plan[15].lat =           0.;

  nplan = sizeof(plan) / sizeof(PLAN);

  eps = 23.4480183837829124;
  st = 12.5354815433383919-10./60.;
  geolon = 0.0;
  geolat = 51.5000000000000000;

  mpSetEvent(&mp, eps, st, geolon, geolat);

  puts("\n\n SPECULUM FOR PRINCE CHARLES (pg 19/1)");
  puts(" -----------------------------------------------------------------------------");
  puts("                LONG     LONG       LAT       DEC       RA         MD       AD");
  puts(" -----------------------------------------------------------------------------");
  mpSetHsys(&mp, mpREGI);
  for(i = 0; i < nplan; i++) 
  {
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);
    deg2zod(plan[i].lon, pLon);   /* longitude   */
    deg2dm(plan[i].lat, pLat);    /* latitude    */
    deg2dm(spec.de, pDec);        /* declination */
   
    mi = spec.md < 0.0 ? 'I' : 'M';
    printf("%s   %s  %7.2f   %s   %s  %7.2f  %7.2f %c  %7.2f\n", 
      plan[i].obj, pLon, plan[i].lon, pLat, pDec, spec.ra, fabs(spec.md), mi, spec.ad);
  }
  printf("\n Press any key to continue...\n ");
  if(getch() == 0x1b) return 0;


  puts("\n\n SPECULUM FOR PRINCE CHARLES (pg 19/2)");
  puts(" -------------------------------------------------");
  puts("               D/N SA       OA/D      AD2    OA/D2");
  puts(" -------------------------------------------------");
  for(i = 0; i < nplan; i++) 
  {
    mpSetHsys(&mp, mpREGI);
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);

    dn = spec.sa < 0.0 ? 'N' : 'D';
    ew = spec.hd < 0.0 ? 'D' : 'A';

    printf("%s    %7.2f %c  %7.2f %c  ", plan[i].obj,
      fabs(spec.sa), dn, spec.oa, ew);

    mpSetHsys(&mp, mpREGI | mpPOLAR);
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);

    printf("%7.2f  %7.2f\n",  -spec.ad, spec.oa);
  }
  printf("\n Press any key to continue...\n ");
  if(getch() == 0x1b) return 0;


  puts("\n\n CAMPANUS-REGIOMONTANUS SPECULUM (pg 48)");
  puts(" --------------------------------------------------------");
  puts("             LONG       RA      DEC         MD         ZD");
  puts(" --------------------------------------------------------");
  mpSetHsys(&mp, mpCAMP);
  for(i = 0; i < nplan; i++) 
  {
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);
    mi = spec.md < 0.0 ? 'I' : 'M';
    zn = spec.zd < 0.0 ? 'N' : 'Z';
    
    printf("%s  %7.2f  %7.2f  %7.2f  %7.2f %c  %7.2f %c\n", plan[i].obj,
      plan[i].lon, spec.ra, spec.de, fabs(spec.md), mi, fabs(spec.zd), zn);
  }
  printf("\n Press any key to continue...\n ");
  if(getch() == 0x1b) return 0;


  puts("\n\n FOR PRINCE CHARLES (pg 49)");
  puts(" -------------------------------------------");
  puts("             POLE        Q        W      CMP");
  puts(" -------------------------------------------");
  for(i = 0; i < nplan; i++) 
  {
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);

    printf("%s  %7.2f  %7.2f  %7.2f  %7.2f\n", plan[i].obj,
      fabs(spec.psi), spec.q, spec.w, spec.mp);
  }
  printf("\n Press any key to continue...\n ");
  if(getch() == 0x1b) return 0;


  puts("\n\n HORIZONTAL SPECULUM FOR PRINCE CHARLES (pg 59)");
  puts(" --------------------------------------------------------------------------");
  puts("               RA         MD        EPD     POLE        Q        W       AZ");
  puts(" --------------------------------------------------------------------------");
  mpSetHsys(&mp, mpCAMP | mpPOLAR);
  for(i = 0; i < nplan; i++) 
  {
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);
    spec.q = spec.psi >= 0. ? spec.q : -spec.q; 
    mi = spec.md < 0.0 ? 'I' : 'M';
    ew = spec.epd < 0.0 ? 'W' : 'E';

    printf("%s  %7.2f  %7.2f %c  %7.2f %c  %7.2f  %7.2f  %7.2f  %7.2f\n", plan[i].obj,
      spec.ra, fabs(spec.md), mi, spec.epd, ew, spec.psi, spec.q, spec.w, spec.az);
  }
  printf("\n Press any key to continue...\n ");
  if(getch() == 0x1b) return 0;


  puts("\n\n PROJECTION SYSTEMS SPECULUM FOR PRINCE CHARLES");
  puts(" ----------------------------------------------------------------------");
  puts("               RA      DEC       ZA       RL     PoMP      AMP      KMP");
  puts(" ----------------------------------------------------------------------");
  for(i = 0; i < nplan; i++)
  {
    mpSetHsys(&mp, mpPORP);
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);
    mpSetHsys(&mp, mpALCA);
    amp = mpCalcPosition(&mp, plan[i].lon, plan[i].lat);
    mpSetHsys(&mp, mpKOCH);
    kmp = mpCalcPosition(&mp, plan[i].lon, plan[i].lat);

    printf("%s  %7.2f  %7.2f  %7.2f  %7.2f  %7.2f  %7.2f  %7.2f\n", plan[i].obj,
      spec.ra, spec.de, spec.w, spec.rl, spec.mp, amp, kmp);
  }
  printf("\n Press any key to continue...\n ");
  if(getch() == 0x1b) return 0;


  puts("\n\n PLACIDUS SPECULUM FOR PRINCE CHARLES (pg 80)");
  puts(" -----------------------------------------------------------------------");
  puts("                L        D       RA       MD       SA     MD/SA      PMP");
  puts(" -----------------------------------------------------------------------");
  mpSetHsys(&mp, mpPLAC);
  for(i = 0; i < nplan; i++)
  {
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);

    printf("%s  %7.2f  %7.2f  %7.2f  %7.2f  %7.2f  %8.5f  %7.2f\n", plan[i].obj,
      plan[i].lon, spec.de, spec.ra, spec.md, spec.sa, spec.pd, spec.mp);
  }
  printf("\n Press any key to continue...\n ");
  if(getch() == 0x1b) return 0;


  puts("\n\n TOPOCENTRIC SPECULUM FOR PRINCE CHARLES (pg 92)");
  puts(" --------------------------------------------------------------");
  puts("               RA      DEC     MD/SA     pole        Q        W");
  puts(" --------------------------------------------------------------");
  mpSetHsys(&mp, mpPOPA);
  for(i = 0; i < nplan; i++)
  {
    mpCalcSpeculum(&mp, plan[i].lon, plan[i].lat, &spec);

    printf("%s  %7.2f  %7.2f  %8.5f  %7.2f  %7.2f  %7.2f\n", plan[i].obj,
      spec.ra, spec.de, fabs(spec.pd), spec.psi, spec.q, spec.w);
  }
  printf("\n Press any key to exit...\n ");
  getch();

  return(0);
}

/* -------------------------------------- */
static double Round(double x, double f)
{
  x *= f;
  if(x >= 0.)
    x = floor(x + .5);
  else
    x = ceil(x - .5);
  return(x /= f);
}

static void deg2zod(double deg, char* s)
{
  static const char sign[12][3] = {"AR", "TA", "GE", "CA", "LE", "VI", "LI", "SC", "SG", "CP", "AQ", "PI"};
  int dd, mm, zz;
  mm = (int)Round(deg * 60.0, 1e0);
  dd = mm / 60;
  mm %= 60;
  zz = dd / 30;
  dd %= 30;
  sprintf(s, "%3d %2s %02d", dd, sign[zz], mm);
}

static void deg2dm(double deg, char* s)
{
  int dd, mm;
  char ns;
  mm = (int)fabs(Round(deg * 60.0, 1e0));
  dd = mm / 60;
  mm %= 60;
  ns = deg < 0.0 ? 'S' : 'N';
  sprintf(s, "%2d %c %02d", dd, ns, mm);
}
